@@ -22,6 +22,8 @@ import six
 # from string import upper
 
 from charmhelpers.core.host import (
+    CompareHostReleases,
+    lsb_release,
     mkdir,
     pwgen,
     write_file
@@ -57,15 +59,6 @@ except ImportError:
     import MySQLdb
 
 
-# NOTE(freyes): Due to skip-name-resolve root@$HOSTNAME account fails when
-# using SET PASSWORD so using UPDATE against the mysql.user table is needed,
-# but changes to this table are not replicated across the cluster, so this
-# update needs to run in all the nodes.
-# More info at http://galeracluster.com/documentation-webpages/userchanges.html
-SQL_UPDATE_PASSWD = ("UPDATE mysql.user SET password = PASSWORD( %s ) "
-                     "WHERE user = %s;")
-
-
 class MySQLSetPasswordError(Exception):
     pass
 
@@ -170,6 +163,25 @@ class MySQLHelper(object):
         finally:
             cursor.close()
 
+    def select(self, sql):
+        """
+        Execute arbitrary SQL select query against the database
+        and return the results.
+
+        :param sql: SQL select query to execute
+        :type sql: string
+        :returns: SQL select query result
+        :rtype: list of lists
+        :raises: MySQLdb.Error
+        """
+        cursor = self.connection.cursor()
+        try:
+            cursor.execute(sql)
+            results = [list(i) for i in cursor.fetchall()]
+        finally:
+            cursor.close()
+        return results
+
     def migrate_passwords_to_leader_storage(self, excludes=None):
         """Migrate any passwords storage on disk to leader storage."""
         if not is_leader():
@@ -311,6 +323,21 @@ class MySQLHelper(object):
                                          'leader settings (%s)') % ex, ex)
 
         try:
+            # NOTE(freyes): Due to skip-name-resolve root@$HOSTNAME account
+            # fails when using SET PASSWORD so using UPDATE against the
+            # mysql.user table is needed, but changes to this table are not
+            # replicated across the cluster, so this update needs to run in
+            # all the nodes. More info at
+            # http://galeracluster.com/documentation-webpages/userchanges.html
+            release = CompareHostReleases(lsb_release()['DISTRIB_CODENAME'])
+            if release < 'bionic':
+                SQL_UPDATE_PASSWD = ("UPDATE mysql.user SET password = "
+                                     "PASSWORD( %s ) WHERE user = %s;")
+            else:
+                # PXC 5.7 (introduced in Bionic) uses authentication_string
+                SQL_UPDATE_PASSWD = ("UPDATE mysql.user SET "
+                                     "authentication_string = "
+                                     "PASSWORD( %s ) WHERE user = %s;")
             cursor.execute(SQL_UPDATE_PASSWD, (new_passwd, username))
             cursor.execute('FLUSH PRIVILEGES;')
             self.connection.commit()
